
using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using WeifenLuo.WinFormsUI;

using ImageLib.Math;
using ImageLib.Imaging;
using ImageLib.Imaging.Filters;
using ImageLib.Imaging.Textures;

namespace StilltoMovie
{
	/// <summary>
	/// Summary description for ImageDoc.
	/// </summary>
	public class ImageDoc : Content
	{
		private System.Drawing.Bitmap backup = null;
		private System.Drawing.Bitmap image = null;
		private string fileName = null;
		private int width;
		private int height;
		private float zoom = 1;
		private IDocumentsHost host = null;

		private bool cropping = false;
		private bool dragging = false;
		private Point start, end, startW, endW;

        private System.Windows.Forms.MainMenu mainMenu;
        private System.Windows.Forms.MenuItem filtersItem;
		private System.Windows.Forms.MenuItem convolutionFiltersItem;
		private System.Windows.Forms.MenuItem meanConvolutionFiltersItem;
		private System.Windows.Forms.MenuItem blurConvolutionFiltersItem;
		private System.Windows.Forms.MenuItem sharpenConvolutionFiltersItem;
        private System.Windows.Forms.MenuItem edgesConvolutionFiltersItem;
		private System.Windows.Forms.MenuItem menuItem12;
        private System.Windows.Forms.MenuItem customConvolutionFiltersItem;
		private System.Windows.Forms.MenuItem hslFiltersItem;
		private System.Windows.Forms.MenuItem brightnessHslFiltersItem;
		private System.Windows.Forms.MenuItem contrastHslFiltersItem;
        private System.Windows.Forms.MenuItem saturationHslFiltersItem;
        private System.Windows.Forms.MenuItem gaussianConvolutionFiltersItem;
		private System.Windows.Forms.MenuItem menuItem33;
        private System.Windows.Forms.MenuItem sharpenExConvolutionFiltersItem;
		private System.ComponentModel.IContainer components;

		// Image property
		public Bitmap Image
		{
			get { return image; }
		}
		// Width property
		public int ImageWidth
		{
			get { return width; }
		}
		// Height property
		public int ImageHeight
		{
			get { return height; }
		}
		// Zoom property
		public float Zoom
		{
			get { return zoom; }
		}
		// FileName property
		// return file name if the document was created from file or null
		public string FileName
		{
			get { return fileName; }
		}


		// Events
		public delegate void SelectionEventHandler(object sender, SelectionEventArgs e);

		public event EventHandler	DocumentChanged;
		//public event EventHandler	ZoomChanged;
		public event SelectionEventHandler	MouseImagePosition;
		public event SelectionEventHandler	SelectionChanged;


		// Constructors
		private ImageDoc(IDocumentsHost host)
		{
			this.host = host;
		}
		// Construct from file
		public ImageDoc(string fileName, IDocumentsHost host) : this(host)
		{
			try
			{
				// load image
				image = (Bitmap) Bitmap.FromFile(fileName);

				// format image
				ImageLib.Imaging.Image.FormatImage(ref image);

				this.fileName = fileName;
			}
			catch (Exception)
			{
				throw new ApplicationException("Failed loading image");
			}

			Init();
		}
		// Construct from image
		public ImageDoc(Bitmap image, IDocumentsHost host) : this(host)
		{
			this.image = image;
			ImageLib.Imaging.Image.FormatImage(ref this.image);

			Init();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
				if (image != null)
				{
					image.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.filtersItem = new System.Windows.Forms.MenuItem();
            this.hslFiltersItem = new System.Windows.Forms.MenuItem();
            this.brightnessHslFiltersItem = new System.Windows.Forms.MenuItem();
            this.contrastHslFiltersItem = new System.Windows.Forms.MenuItem();
            this.saturationHslFiltersItem = new System.Windows.Forms.MenuItem();
            this.convolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.meanConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.blurConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.sharpenConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.edgesConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.customConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.menuItem33 = new System.Windows.Forms.MenuItem();
            this.gaussianConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.sharpenExConvolutionFiltersItem = new System.Windows.Forms.MenuItem();
            this.SuspendLayout();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.filtersItem});
            // 
            // filtersItem
            // 
            this.filtersItem.Index = 0;
            this.filtersItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.hslFiltersItem,
            this.convolutionFiltersItem});
            this.filtersItem.MergeOrder = 1;
            this.filtersItem.Text = "PreProcessing";
            // 
            // hslFiltersItem
            // 
            this.hslFiltersItem.Index = 0;
            this.hslFiltersItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.brightnessHslFiltersItem,
            this.contrastHslFiltersItem,
            this.saturationHslFiltersItem});
            this.hslFiltersItem.Text = "&HSL Color space";
            // 
            // brightnessHslFiltersItem
            // 
            this.brightnessHslFiltersItem.Index = 0;
            this.brightnessHslFiltersItem.Text = "&Brightness";
            this.brightnessHslFiltersItem.Click += new System.EventHandler(this.brightnessHslFiltersItem_Click);
            // 
            // contrastHslFiltersItem
            // 
            this.contrastHslFiltersItem.Index = 1;
            this.contrastHslFiltersItem.Text = "&Contrast";
            this.contrastHslFiltersItem.Click += new System.EventHandler(this.contrastHslFiltersItem_Click);
            // 
            // saturationHslFiltersItem
            // 
            this.saturationHslFiltersItem.Index = 2;
            this.saturationHslFiltersItem.Text = "&Saturation";
            this.saturationHslFiltersItem.Click += new System.EventHandler(this.saturationHslFiltersItem_Click);
            // 
            // convolutionFiltersItem
            // 
            this.convolutionFiltersItem.Index = 1;
            this.convolutionFiltersItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.meanConvolutionFiltersItem,
            this.blurConvolutionFiltersItem,
            this.sharpenConvolutionFiltersItem,
            this.edgesConvolutionFiltersItem,
            this.menuItem12,
            this.customConvolutionFiltersItem,
            this.menuItem33,
            this.gaussianConvolutionFiltersItem,
            this.sharpenExConvolutionFiltersItem});
            this.convolutionFiltersItem.Text = "Co&nvolution && Correlation";
            // 
            // meanConvolutionFiltersItem
            // 
            this.meanConvolutionFiltersItem.Index = 0;
            this.meanConvolutionFiltersItem.Text = "&Mean";
            this.meanConvolutionFiltersItem.Click += new System.EventHandler(this.meanConvolutionFiltersItem_Click);
            // 
            // blurConvolutionFiltersItem
            // 
            this.blurConvolutionFiltersItem.Index = 1;
            this.blurConvolutionFiltersItem.Text = "&Blur";
            this.blurConvolutionFiltersItem.Click += new System.EventHandler(this.blurConvolutionFiltersItem_Click);
            // 
            // sharpenConvolutionFiltersItem
            // 
            this.sharpenConvolutionFiltersItem.Index = 2;
            this.sharpenConvolutionFiltersItem.Text = "&Sharpen";
            this.sharpenConvolutionFiltersItem.Click += new System.EventHandler(this.sharpenConvolutionFiltersItem_Click);
            // 
            // edgesConvolutionFiltersItem
            // 
            this.edgesConvolutionFiltersItem.Index = 3;
            this.edgesConvolutionFiltersItem.Text = "&Edges";
            this.edgesConvolutionFiltersItem.Click += new System.EventHandler(this.edgesConvolutionFiltersItem_Click);
            // 
            // menuItem12
            // 
            this.menuItem12.Index = 4;
            this.menuItem12.Text = "-";
            // 
            // customConvolutionFiltersItem
            // 
            this.customConvolutionFiltersItem.Index = 5;
            this.customConvolutionFiltersItem.Text = "&Morphology";
            this.customConvolutionFiltersItem.Click += new System.EventHandler(this.customConvolutionFiltersItem_Click);
            // 
            // menuItem33
            // 
            this.menuItem33.Index = 6;
            this.menuItem33.Text = "-";
            // 
            // gaussianConvolutionFiltersItem
            // 
            this.gaussianConvolutionFiltersItem.Index = 7;
            this.gaussianConvolutionFiltersItem.Text = "&Gaussian";
            this.gaussianConvolutionFiltersItem.Click += new System.EventHandler(this.gaussianConvolutionFiltersItem_Click);
            // 
            // sharpenExConvolutionFiltersItem
            // 
            this.sharpenExConvolutionFiltersItem.Index = 8;
            this.sharpenExConvolutionFiltersItem.Text = "Sharpen Ex";
            this.sharpenExConvolutionFiltersItem.Click += new System.EventHandler(this.sharpenExConvolutionFiltersItem_Click);
            // 
            // ImageDoc
            // 
            this.AllowedStates = WeifenLuo.WinFormsUI.ContentStates.Document;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(744, 465);
            this.Menu = this.mainMenu;
            this.Name = "ImageDoc";
            this.Text = "Image";
            this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.ImageDoc_MouseUp);
            this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ImageDoc_MouseDown);
            this.MouseLeave += new System.EventHandler(this.ImageDoc_MouseLeave);
            this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.ImageDoc_MouseMove);
            this.ResumeLayout(false);

		}
		#endregion

		// Init the document
		private void Init()
		{
			// init components
			InitializeComponent();

			// form style
			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.DoubleBuffer | ControlStyles.ResizeRedraw, true);

			// init scroll bars
			this.AutoScroll = true;

			UpdateSize();
		}

		// Execute command
		public void ExecuteCommand(ImageDocCommands cmd)
		{
			switch (cmd)
			{
							
				case ImageDocCommands.Convolution:	// convolution
					Convolution();
					break;
				case ImageDocCommands.Brightness:	// adjust brightness
					Brightness();
					break;
				case ImageDocCommands.Contrast:		// modify contrast
					Contrast();
					break;
				case ImageDocCommands.Saturation:	// adjust saturation
					Saturation();
					break;
				case ImageDocCommands.Fourier:		// fourier transformation
					ForwardFourierTransformation();
					break;
			}
		}

		// Update document and notify client about changes
		private void UpdateNewImage()
		{
			// update size
			UpdateSize();
			// repaint
			Invalidate();

			// notify host
			if (DocumentChanged != null)
				DocumentChanged(this, null);
		}

		

		// Update document size 
		private void UpdateSize()
		{
			// image dimension
			width = image.Width;
			height = image.Height;

			// scroll bar size
			this.AutoScrollMinSize = new Size((int)(width * zoom), (int)(height * zoom));
		}

		// Paint image
		protected override void OnPaint(PaintEventArgs e)
		{
			if (image != null)
			{
				Graphics	g = e.Graphics;
				Rectangle	rc = ClientRectangle;
				Pen			pen = new Pen(Color.FromArgb(0, 0, 0));

				int			width = (int) (this.width * zoom);
				int			height = (int) (this.height * zoom);
				int			x = (rc.Width < width) ? this.AutoScrollPosition.X : (rc.Width - width) / 2;
				int			y = (rc.Height < height) ? this.AutoScrollPosition.Y : (rc.Height - height) / 2;

				// draw rectangle around the image
				g.DrawRectangle(pen, x - 1, y - 1, width + 1, height + 1);

				// set nearest neighbor interpolation to avoid image smoothing
				g.InterpolationMode = InterpolationMode.NearestNeighbor;

				// draw image
				g.DrawImage(image, x, y, width, height);

				pen.Dispose();
			}
		}

		// Mouse click
		protected override void OnClick(EventArgs e)
		{
			Focus();
		}

		// Apply filter on the image
		private void ApplyFilter(IFilter filter)
		{
			try
			{
				// set wait cursor
				this.Cursor = Cursors.WaitCursor;

				// apply filter to the image
				Bitmap newImage = filter.Apply(image);

				if (host.CreateNewDocumentOnChange)
				{
					// open new image in new document
					host.NewDocument(newImage);
				}
				else
				{
					if (host.RememberOnChange)
					{
						// backup current image
						if (backup != null)
							backup.Dispose();

						backup = image;
					}
					else
					{
						// release current image
						image.Dispose();
					}

					image = newImage;

					// update
					UpdateNewImage();
				}
			}
			catch (ArgumentException)
			{
				MessageBox.Show("Selected filter can not be applied to the image", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			finally
			{
				// restore cursor
				this.Cursor = Cursors.Default;
			}
		}

		// on "Image" item popup
		private void imageItem_Popup(object sender, System.EventArgs e)
		{
			//this.backImageItem.Enabled = (backup != null);
			//this.cropImageItem.Checked = cropping;
		}

		// Restore image to previous
		private void backImageItem_Click(object sender, System.EventArgs e)
		{
			if (backup != null)
			{
				// release current image
				image.Dispose();
				// restore
				image = backup;
				backup = null;

				// update
				UpdateNewImage();
			}
		}
	
		// Adjust brighness using HSL
		private void Brightness()
		{
			if (image.PixelFormat != PixelFormat.Format24bppRgb)
			{
				MessageBox.Show("Brightness filter using HSL color space is available for color images only", "Message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			BrightnessForm form = new BrightnessForm();
			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// On "Filters->HSL Color space->Brighness" menu item click
		private void brightnessHslFiltersItem_Click(object sender, System.EventArgs e)
		{
			Brightness();
		}

		// Modify contrast
		private void Contrast()
		{
			if (image.PixelFormat != PixelFormat.Format24bppRgb)
			{
				MessageBox.Show("Contrast filter using HSL color space is available for color images only", "Message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			ContrastForm form = new ContrastForm();
			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// On "Filters->HSL Color space->Contrast" menu item click
		private void contrastHslFiltersItem_Click(object sender, System.EventArgs e)
		{
			Contrast();
		}

		// Adjust saturation using HSL
		private void Saturation()
		{
			if (image.PixelFormat != PixelFormat.Format24bppRgb)
			{
				MessageBox.Show("Saturation filter using HSL color space is available for color images only", "Message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			SaturationForm form = new SaturationForm();
			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// On "Filters->HSL Color space->Saturation" menu item click
		private void saturationHslFiltersItem_Click(object sender, System.EventArgs e)
		{
			Saturation();
		}	

		

		// Mean
		private void meanConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			ApplyFilter(new Mean());
		}

		// Blur
		private void blurConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			ApplyFilter(new Blur());
		}

		// Gaussian smoothing
		private void gaussianConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			GaussianForm form = new GaussianForm();

			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// Extended sharpening
		private void sharpenExConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			SharpenExForm form = new SharpenExForm();

			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// Sharpen
		private void sharpenConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			ApplyFilter(new Sharpen());
		}

		// Edges
		private void edgesConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			ApplyFilter(new Edges());
		}

		// Custom convolution filter
		private void Convolution()
		{
			ConvolutionForm	form = new ConvolutionForm();

			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// On "Filters->Convolution & Correlation->Custom" menu item click
		private void customConvolutionFiltersItem_Click(object sender, System.EventArgs e)
		{
			Convolution();
		}	


		// Canny edge detector
		private void cannyEdgeFiltersItem_Click(object sender, System.EventArgs e)
		{
			CannyDetectorForm form = new CannyDetectorForm();

			form.Image = image;

			if (form.ShowDialog() == DialogResult.OK)
			{
				ApplyFilter(form.Filter);
			}
		}

		// Fourier transformation
		private void ForwardFourierTransformation()
		{
			if ((!ImageLib.Math.Tools.IsPowerOf2(width)) ||
				(!ImageLib.Math.Tools.IsPowerOf2(height)))
			{
				MessageBox.Show("Fourier trasformation can be applied to an image with width and height of power of 2", "Message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			ComplexImage	cImage = ComplexImage.FromBitmap(image);

			cImage.ForwardFourierTransform();
			host.NewDocument(cImage);
		}

		// On "Filters->Fourier Transformation" click
		private void fourierFiltersItem_Click(object sender, System.EventArgs e)
		{
			ForwardFourierTransformation();
		}

		// Calculate image and screen coordinates of the point
		private void GetImageAndScreenPoints(Point point, out Point imgPoint, out Point screenPoint)
		{
			Rectangle	rc = this.ClientRectangle;
			int			width = (int) (this.width * zoom);
			int			height = (int) (this.height * zoom);
			int			x = (rc.Width < width) ? this.AutoScrollPosition.X : (rc.Width - width) / 2;
			int			y = (rc.Height < height) ? this.AutoScrollPosition.Y : (rc.Height - height) / 2;

			int			ix = Math.Min(Math.Max(x, point.X), x + width - 1);
			int			iy = Math.Min(Math.Max(y, point.Y), y + height - 1);

			ix = (int)((ix - x) / zoom);
			iy = (int)((iy - y) / zoom);

			// image point
			imgPoint = new Point(ix, iy);
			// screen point
			screenPoint = this.PointToScreen(new Point((int)(ix * zoom + x), (int)(iy * zoom + y)));
		}

		// Normalize points so, that pt1 becomes top-left point of rectangle
		// and pt2 becomes right-bottom
		private void NormalizePoints(ref Point pt1, ref Point pt2)
		{
			Point t1 = pt1;
			Point t2 = pt2;

			pt1.X = Math.Min(t1.X, t2.X);
			pt1.Y = Math.Min(t1.Y, t2.Y);
			pt2.X = Math.Max(t1.X, t2.X);
			pt2.Y = Math.Max(t1.Y, t2.Y);
		}

		// Draw selection rectangle
		private void DrawSelectionFrame(Graphics g)
		{
			Point	sp = startW;
			Point	ep = endW;

			// Normalize points
			NormalizePoints(ref sp, ref ep);
			// Draw reversible frame
			ControlPaint.DrawReversibleFrame(new Rectangle(sp.X, sp.Y, ep.X - sp.X + 1, ep.Y - sp.Y + 1), Color.White, FrameStyle.Dashed);
		}

	

		// On mouse down
		private void ImageDoc_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				// turn off cropping mode
				if (!dragging)
				{
					cropping = false;
					this.Cursor = Cursors.Default;
				}
			}
			else if (e.Button == MouseButtons.Left)
			{
				if (cropping)
				{
					// start dragging
					dragging = true;
					// set mouse capture
					this.Capture = true;

					// get selection start point
					GetImageAndScreenPoints(new Point(e.X, e.Y), out start, out startW);

					// end point is the same as start
					end = start;
					endW = startW;
					
					// draw frame
					Graphics g = this.CreateGraphics();
					DrawSelectionFrame(g);
					g.Dispose();
				}
			}
		}

		// On mouse up
		private void ImageDoc_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (dragging)
			{
				// stop dragging and cropping
				dragging = cropping = false;
				// release capture
				this.Capture = false;
				// set default mouse pointer
				this.Cursor = Cursors.Default;

				// erase frame
				Graphics g = this.CreateGraphics();
				DrawSelectionFrame(g);
				g.Dispose();

				// normalize start and end points
				NormalizePoints(ref start, ref end);

				// crop tge image
				ApplyFilter(new Crop(new Rectangle(start.X, start.Y, end.X - start.X + 1, end.Y - start.Y + 1)));
			}
		}

		// On mouse move
		private void ImageDoc_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (dragging)
			{

				Graphics g = this.CreateGraphics();

				// erase frame
				DrawSelectionFrame(g);

				// get selection end point
				GetImageAndScreenPoints(new Point(e.X, e.Y), out end, out endW);

				// draw frame
				DrawSelectionFrame(g);

				g.Dispose();

				if (SelectionChanged != null)
				{
					Point	sp = start;
					Point	ep = end;

					// normalize start and end points
					NormalizePoints(ref sp, ref ep);

					SelectionChanged(this, new SelectionEventArgs(
						sp, new Size(ep.X - sp.X + 1, ep.Y - sp.Y + 1)));
				}
			}
			else
			{
				if (MouseImagePosition != null)
				{
					Rectangle	rc = this.ClientRectangle;
					int			width = (int) (this.width * zoom);
					int			height = (int) (this.height * zoom);
					int			x = (rc.Width < width) ? this.AutoScrollPosition.X : (rc.Width - width) / 2;
					int			y = (rc.Height < height) ? this.AutoScrollPosition.Y : (rc.Height - height) / 2;

					if ((e.X >= x) && (e.Y >= y) &&
						(e.X < x + width) && (e.Y < y + height))
					{
						// mouse is over the image
						MouseImagePosition(this, new SelectionEventArgs(
							new Point((int)((e.X - x) / zoom), (int)((e.Y - y) / zoom))));
					}
					else
					{
						// mouse is outside image region
						MouseImagePosition(this, new SelectionEventArgs(new Point(-1, -1)));
					}
				}
			}
		}

		// On mouse leave
		private void ImageDoc_MouseLeave(object sender, System.EventArgs e)
		{
			if ((!dragging) && (MouseImagePosition != null))
			{
				MouseImagePosition(this, new SelectionEventArgs(new Point(-1, -1)));
			}
		}

      
	}

	// Selection arguments
	public class SelectionEventArgs : EventArgs
	{
		private Point	location;
		private Size	size;

		// Constructors
		public SelectionEventArgs(Point location)
		{
			this.location = location;
		}
		public SelectionEventArgs(Point location, Size size)
		{
			this.location = location;
			this.size = size;
		}

		// Location property
		public Point Location
		{
			get { return location; }
		}
		// Size property
		public Size Size
		{
			get { return size; }
		}
	}

	// Commands
	public enum ImageDocCommands
	{
		Clone,
		Crop,
		ZoomIn,
		ZoomOut,
		ZoomOriginal,
		FitToSize,
		Levels,
		Grayscale,
		Threshold,
		Morphology,
		Convolution,
		Resize,
		Rotate,
		Brightness,
		Contrast,
		Saturation,
		Fourier
	}
}
